@@ -33,17 +33,21 @@ class NestedDict(object):
 
     def __getitem__(self, key):
         rv = self.data[key]
-        if type(rv) == dict:
-            rv = NestedDict(rv)
-            self.data[key] = rv
         if type(rv) is type(self) and not rv.parent:
             object.__setattr__(rv, "parent", self)
             return rv
         return rv
 
     def __getattr__(self, key):
+        keys = key.split(".")
         try:
-            return self[key]
+            r = self[keys[0]]
+            if not len(keys[1:]):
+                return r
+            if type(r) not in (dict, NestedDict):
+                raise AttributeError("No attribute found for %s" % keys[1:])
+
+            return getattr(r, ".".join(keys[1:]))
         except KeyError as e:
             raise AttributeError(e)
 
@@ -54,12 +58,16 @@ class NestedDict(object):
             return default
 
     def __setattr__(self, key, value):
-        r = d = {}
-        keys = key.split(".")
-        for k in keys[:-1]:
-            d = d.setdefault(k, {})
-        d[keys[-1]] = value
-        self.update(r)
+        self.update({key: value})
+
+    def __setitem__(self, key, value):
+        self.update({key: value})
+
+    def setdefault(self, key, default=None):
+        if key in self:
+            return self[key]
+        self[key] = default
+        return default
 
     def has(self, key):
         return key in self.data
@@ -68,18 +76,21 @@ class NestedDict(object):
         return self.data
 
     def update(self, other):
-        if type(other) in (dict, NestedDict):
-            for (key, value) in other.items():
-                if key in self.data and \
-                    type(self[key]) in (dict, NestedDict) and \
-                        type(value) in (dict, NestedDict):
-                    self.data[key] = NestedDict(self[key])
-                    object.__setattr__(self.data[key], "parent", self)
-                    self.data[key].update(value)
+        for (key, value) in other.items():
+            value = NestedDict(value) if type(value) is dict else value
+            if key in self.data and \
+                type(self[key]) in (dict, NestedDict) and \
+                    type(value) in (dict, NestedDict):
+                self.data[key] = NestedDict(self[key])
+                object.__setattr__(self.data[key], "parent", self)
+                self.data[key].update(value)
+            else:
+                if '.' in key:
+                    key, remainder = key.split('.', 1)
+                    d = NestedDict({remainder: value})
+                    self.data[key] = d
                 else:
                     self.data[key] = value
-        else:
-            self.data.update(other)
 
     def lookup(self, path, default=None):
         b = self.data.copy()
